@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -265,7 +262,7 @@ bool PowerMac7_3_CPUCoolingCtrlLoop::choosePIDDataset( const OSDictionary * ctrl
 	int i, count, cmp;
 	const OSArray * tmp_array;
 	OSArray * metaStateArray;
-	OSDictionary * eepromDataset, * plistDataset, * chosenDataset;
+	OSDictionary * eepromDataset, * plistDataset = NULL, * chosenDataset;
 	const OSData * binID, * plistBinID;
 
 	if (!ctrlLoopDict) return(false);
@@ -1021,15 +1018,12 @@ ControlValue PowerMac7_3_CPUCoolingCtrlLoop::calculateNewTarget( void ) const
 		else if (newTarget > outputMax)
 			newTarget = outputMax;
 
-/*
+
 #ifdef CTRLLOOP_DEBUG
 		if (timerCallbackActive)
 		{
 			const OSString * tempDesc;
-			const OSNumber * val;
-			val = slewControl->fetchCurrentValue();
-			slewControl->setCurrentValue( val );
-			val->release();
+			if (slewControl) slewControl->setCurrentValue( slewControl->fetchCurrentValue() );
 #endif
 			CTRLLOOP_DLOG("%s"
 						" powerSum=%08lX (%ld)"
@@ -1046,8 +1040,8 @@ ControlValue PowerMac7_3_CPUCoolingCtrlLoop::calculateNewTarget( void ) const
 						" G_d=%08lX"
 						" dProd=%016llX (%lld)"
 						" Res=%016llX"
-						" Out=%lu"
-						" Spd=%0u\n",
+						" Out=%ld"
+						" Spd=%0lu\n",
 						(tempDesc = OSDynamicCast( OSString, infoDict->getObject(kIOPPluginThermalGenericDescKey))) != NULL ?
 								tempDesc->getCStringNoCopy() : "Unknown CtrlLoop",
 						rRaw, rRaw >> 16,
@@ -1064,12 +1058,12 @@ ControlValue PowerMac7_3_CPUCoolingCtrlLoop::calculateNewTarget( void ) const
 						G_d,
 						dProd, dProd >> 36,
 						accum,
-						uResult,
-						slewControl->getCurrentValue()->unsigned8BitValue() );
+						result,
+						slewControl ? slewControl->getCurrentValue() : 0xFFFFFFFF );
 #ifdef CTRLLOOP_DEBUG
 		}
 #endif
-*/
+
 
 	}
 
