@@ -1,33 +1,31 @@
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc.  All rights reserved.
- *
+ * Copyright (c) 2003-2004 Apple Computer, Inc.  All rights reserved.
  *
+ *  File: $Id: RackMac3_1_CPUFanCtrlLoop.cpp,v 1.8 2004/03/18 02:18:52 eem Exp $
  */
 
+
 #include "IOPlatformPluginDefs.h"
 #include "IOPlatformPluginSymbols.h"
 #include "RackMac3_1_PlatformPlugin.h"
@@ -173,7 +171,7 @@ bool RackMac3_1_CPUFanCtrlLoop::choosePIDDataset( const OSDictionary * ctrlLoopD
     int i, count, cmp;
     const OSArray * tmp_array;
     OSArray * metaStateArray;
-    OSDictionary * eepromDataset, * plistDataset, * chosenDataset;
+    OSDictionary * eepromDataset, * plistDataset = NULL, * chosenDataset;
     const OSData * binID, * plistBinID;
 
     if (!ctrlLoopDict) return(false);
