@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -193,7 +190,7 @@ bool MacRISC4CPU::start(IOService *provider)
 	
 	doSleep = false;
   
-    // Get the "flush-on-lock" property from the first cpu node.  "flush-on-lock" mean do no "nap" the processor.
+    // Get the "flush-on-lock" property from the first cpu node.  "flush-on-lock" mean do not "nap" the processor.
     flushOnLock = false;
     cpusRegEntry = fromPath("/cpus/@0", gIODTPlane);
     if (cpusRegEntry == 0) return false;
@@ -342,7 +339,7 @@ bool MacRISC4CPU::start(IOService *provider)
 	}
 
     if (physCPU < numCPUs)
-    {		
+    {
         processor_info.cpu_id           = (cpu_id_t)this;
         processor_info.boot_cpu         = bootCPU;
         processor_info.start_paddr      = 0x0100;
@@ -350,6 +347,25 @@ bool MacRISC4CPU::start(IOService *provider)
         processor_info.supports_nap     = !flushOnLock;
         processor_info.time_base_enable = MacRISC4CPU::sEnableCPUTimeBase;
 
+		OSData*							powerTuneData;
+
+		// power_mode_0 and power_mode_1 are defined in a newer version of ppc/machine_routines.h.
+
+		if ( ( powerTuneData = OSDynamicCast( OSData, cpusRegEntry->getProperty( "power-mode-data" ) ) ) != NULL )
+		{
+			uint32_t*					powerTuneEntry;
+
+			powerTuneEntry = ( uint32_t * ) powerTuneData->getBytesNoCopy();
+
+			// Only copy power_mode_1 if it exists.
+
+			if ( powerTuneData->getLength() >= ( 2 * sizeof( uint32_t ) ) )
+				{
+				processor_info.power_mode_0 = powerTuneEntry[ 0 ];
+				processor_info.power_mode_1 = powerTuneEntry[ 1 ];
+				}
+		}
+
 		// Register this CPU with mach.
 		kprintf("MacRISC4CPU::start(%ld) - registering with mach\n", getCPUNumber());
 		result = ml_processor_register(&processor_info, &machProcessor,	&ipi_handler);
